/*
 * Copyright (c) 1998, 2002 Sandro Sigala.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <fxkeys.h>

#include "mainwindow.h"
#include "gamewidget.h"
#include "version.h"
#include "paths.h"

#include "fxzacman.xpm"
#include "fxzacman_mini.xpm"

FXDEFMAP(MainWindow) MainWindowMap[] = {
  FXMAPFUNC(SEL_COMMAND,   MainWindow::ID_NEW, 	    MainWindow::onCmdNew),
  FXMAPFUNC(SEL_COMMAND,   MainWindow::ID_PAUSE,    MainWindow::onCmdPause),
  FXMAPFUNC(SEL_UPDATE,    MainWindow::ID_PAUSE,    MainWindow::onUpdPause),
  FXMAPFUNC(SEL_COMMAND,   MainWindow::ID_HIGHSCORE,MainWindow::onCmdHighScore),
  FXMAPFUNC(SEL_COMMAND,   MainWindow::ID_ICONIFY,  MainWindow::onCmdIconify),
  FXMAPFUNC(SEL_SIGNAL,    MainWindow::ID_QUIT,     MainWindow::onCmdQuit),
  FXMAPFUNC(SEL_COMMAND,   MainWindow::ID_QUIT,     MainWindow::onCmdQuit),
  FXMAPFUNC(SEL_COMMAND,   MainWindow::ID_SKILL,    MainWindow::onCmdSkill),
  FXMAPFUNC(SEL_COMMAND,   MainWindow::ID_SOUND,    MainWindow::onCmdSound),
  FXMAPFUNC(SEL_COMMAND,   MainWindow::ID_ABOUT,    MainWindow::onCmdAbout),
  FXMAPFUNC(SEL_COMMAND,   MainWindow::ID_UP,  	    MainWindow::onCmdUp),
  FXMAPFUNC(SEL_COMMAND,   MainWindow::ID_DOWN,     MainWindow::onCmdDown),
  FXMAPFUNC(SEL_COMMAND,   MainWindow::ID_LEFT,     MainWindow::onCmdLeft),
  FXMAPFUNC(SEL_COMMAND,   MainWindow::ID_RIGHT,    MainWindow::onCmdRight),
  FXMAPFUNC(SEL_TIMEOUT,   MainWindow::ID_TIMER,    MainWindow::onTimer),
};

FXIMPLEMENT(MainWindow, FXMainWindow, MainWindowMap, ARRAYNUMBER(MainWindowMap))

MainWindow::MainWindow()
{
}

MainWindow::MainWindow(FXApp *app)
    : FXMainWindow(app, "FX-Zacman", 0, 0, DECOR_TITLE|DECOR_MINIMIZE|DECOR_RESIZE|DECOR_CLOSE|DECOR_BORDER|DECOR_MENU, 0, 0, 300, 300)
{
	bigicon = new FXXPMIcon(getApp(), fxzacman_xpm);
  	smallicon = new FXXPMIcon(getApp(), fxzacman_mini_xpm);
 	setIcon(bigicon);
    	setMiniIcon(smallicon);

       	FXMenubar *menubar;

        FXToolbarShell *dragshell1 = new FXToolbarShell(this, FRAME_RAISED);
        menubar = new FXMenubar(this, dragshell1, FRAME_RAISED|LAYOUT_SIDE_TOP|LAYOUT_FILL_X);
        new FXToolbarGrip(menubar, menubar, FXMenubar::ID_TOOLBARGRIP);

       	FXStatusbar *statusbar;
  	statusbar = new FXStatusbar(this, LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X|STATUSBAR_WITH_DRAGCORNER|FRAME_RAISED);

        filemenu = new FXMenuPane(this);
        new FXMenuCommand(filemenu, "&New\tF2\tStart a new game.", 0, this, ID_NEW);
        new FXMenuCommand(filemenu, "&Pause\tP\tPause the game.", 0, this, ID_PAUSE);
  	new FXMenuSeparator(filemenu);
        new FXMenuCommand(filemenu, "Show &highscores...\t\tShow highscores.", 0, this, ID_HIGHSCORE);
  	new FXMenuSeparator(filemenu);
        new FXMenuCommand(filemenu, "&Quit\tAlt-X\tClose this program.", 0, this, ID_QUIT);
        new FXMenuTitle(menubar, "&Game", 0, filemenu);

        optionsmenu = new FXMenuPane(this);
       	FXMenuCascade *skillcascade;
	skillcascade = new FXMenuCascade(optionsmenu, "&Skill level\t\tSelect the game skill level.");
	new FXMenuSeparator(optionsmenu);
        new FXMenuCommand(optionsmenu, "&Sound\t\tEnable/disable sound.", 0, &sound_target, FXDataTarget::ID_VALUE);
        new FXMenuTitle(menubar, "&Options", 0, optionsmenu);

        skillmenu = new FXMenuPane(this);
        skillcascade->setMenu(skillmenu);
        new FXMenuCommand(skillmenu, "&Beginner\t\tStart a new game with the beginner skill level.", 0, &skill_target, FXDataTarget::ID_OPTION+0);
        new FXMenuCommand(skillmenu, "&Intermediate\t\tStart a new game with the intermediate skill level.", 0, &skill_target, FXDataTarget::ID_OPTION+1);
        new FXMenuCommand(skillmenu, "&Advanced\t\tStart a new game with the advanced skill level.", 0, &skill_target, FXDataTarget::ID_OPTION+2);
        new FXMenuCommand(skillmenu, "&Expert\t\tStart a new game with the expert skill level.", 0, &skill_target, FXDataTarget::ID_OPTION+3);

        helpmenu = new FXMenuPane(this);
        new FXMenuCommand(helpmenu, "&About...\t\tShow information about this program.", 0, this, ID_ABOUT);
        new FXMenuTitle(menubar, "&?", 0, helpmenu);

       	game = new GameWidget(this, this, ID_GAME, LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y,
        	0,0,default_game_width,default_game_height);

  	pills_label = new FXLabel(statusbar, " 0 ", 0, FRAME_SUNKEN|LAYOUT_RIGHT|LAYOUT_CENTER_Y);
 	new FXLabel(statusbar," Pills left:",0,LAYOUT_RIGHT|LAYOUT_CENTER_Y);
  	time_label = new FXLabel(statusbar, " 0 ", 0, FRAME_SUNKEN|LAYOUT_RIGHT|LAYOUT_CENTER_Y);
 	new FXLabel(statusbar," Time:",0,LAYOUT_RIGHT|LAYOUT_CENTER_Y);

	getApp()->reg().read();
	game->engine.sounds = getApp()->reg().readUnsignedEntry("Settings", "Sounds", 1);
	skill = getApp()->reg().readUnsignedEntry("Settings", "Skill", 0);
	skill = skill % 5; 	// Force 0 to 4

	sound_target.connect(game->engine.sounds);
  	sound_target.setTarget(this);
  	sound_target.setSelector(ID_SOUND);

  	skill_target.connect(skill);
  	skill_target.setTarget(this);
  	skill_target.setSelector(ID_SKILL);
	game->engine.newGame((Skill)skill);

	FXAccelTable *t = getAccelTable();
	if (t) {
    		t->addAccel(KEY_Escape, this, MKUINT(ID_ICONIFY, SEL_COMMAND));
    		t->addAccel(KEY_Up,     this, MKUINT(ID_UP,      SEL_COMMAND));
    		t->addAccel(KEY_Down,   this, MKUINT(ID_DOWN,    SEL_COMMAND));
    		t->addAccel(KEY_Left,   this, MKUINT(ID_LEFT,    SEL_COMMAND));
    		t->addAccel(KEY_Right,  this, MKUINT(ID_RIGHT,   SEL_COMMAND));
    	}

	lastkeytime = 0;

	timerhandle = getApp()->addTimeout(100, this, ID_TIMER);
}

MainWindow::~MainWindow()
{
	delete filemenu;
	delete optionsmenu;
	delete skillmenu;
	delete helpmenu;
	getApp()->removeTimeout(timerhandle);

	getApp()->reg().writeUnsignedEntry("Settings", "Skill", game->engine.game_skill);
	getApp()->reg().writeUnsignedEntry("Settings", "Sounds", game->engine.sounds);
	getApp()->reg().write();
}

void MainWindow::create()
{
  	FXMainWindow::create();
}

long MainWindow::onCmdNew(FXObject *, FXSelector, void *)
{
	game->engine.newGame((Skill)game->engine.game_skill);
	return 1;
}

long MainWindow::onCmdPause(FXObject *, FXSelector, void *)
{
	if (game->engine.status == Status_Playing)
		game->engine.paused = !game->engine.paused;
	return 1;
}

long MainWindow::onUpdPause(FXObject *sender, FXSelector, void *)
{
	FXint checked = game->engine.status == Status_Playing && game->engine.paused;
	sender->handle(this, MKUINT(ID_SETINTVALUE, SEL_COMMAND), (void*)&checked);
  	FXuint enabled = (game->engine.status == Status_Playing) ? ID_ENABLE : ID_DISABLE;
   	sender->handle(this, MKUINT(enabled, SEL_COMMAND), 0);
	return 1;
}

long MainWindow::onCmdHighScore(FXObject *, FXSelector, void *)
{
	FXDialogBox dlg(this, "FX-Zacman Highscores", DECOR_TITLE|DECOR_BORDER,
 		0,0,0,0, 0,0,0,0, 0,0);

  	FXVerticalFrame *contents =
        	new FXVerticalFrame(&dlg, LAYOUT_SIDE_LEFT|LAYOUT_FILL_X|LAYOUT_FILL_Y,
       	 		0,0,0,0, 10,10,10,10, 0,0);

	FXHorizontalFrame *h0 =
        	new FXHorizontalFrame(contents, LAYOUT_SIDE_LEFT|LAYOUT_FILL_X|LAYOUT_FILL_Y,
    	 		0,0,0,0, 5,5,5,5, 0,0);
	new FXLabel(h0, "Level", 0, LAYOUT_FIX_WIDTH|LAYOUT_FILL_Y|JUSTIFY_LEFT,
	 		0,0,80,0, 0,0,0,0);
	new FXLabel(h0, "Name", 0, LAYOUT_FIX_WIDTH|LAYOUT_FILL_Y|JUSTIFY_LEFT,
	 		0,0,120,0, 0,0,0,0);
	new FXLabel(h0, "Time", 0, LAYOUT_FILL_X);

      	new FXHorizontalSeparator(contents, SEPARATOR_GROOVE|LAYOUT_FILL_X);

	FXHorizontalFrame *h1 =
        	new FXHorizontalFrame(contents, LAYOUT_SIDE_LEFT|LAYOUT_FILL_X|LAYOUT_FILL_Y,
    	 		0,0,0,0, 5,5,5,5, 0,0);
	new FXLabel(h1, "Beginner", 0, LAYOUT_FIX_WIDTH|LAYOUT_FILL_Y|JUSTIFY_LEFT,
	 		0,0,80,0, 0,0,0,0);
	FXString n = getApp()->reg().readStringEntry("Scores", "BeginnerName", "Noname");
	new FXLabel(h1, n, 0, LAYOUT_FIX_WIDTH|LAYOUT_FILL_Y|JUSTIFY_LEFT,
	 		0,0,120,0, 0,0,0,0);
	int v = getApp()->reg().readUnsignedEntry("Scores", "BeginnerTime", 9999);
	new FXLabel(h1, FXStringVal(v), 0, FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X);

	FXHorizontalFrame *h2 =
        	new FXHorizontalFrame(contents, LAYOUT_SIDE_LEFT|LAYOUT_FILL_X|LAYOUT_FILL_Y,
    	 		0,0,0,0, 5,5,5,5, 0,0);
	new FXLabel(h2, "Intermediate", 0, LAYOUT_FIX_WIDTH|LAYOUT_FILL_Y|JUSTIFY_LEFT,
	 		0,0,80,0, 0,0,0,0);
	n = getApp()->reg().readStringEntry("Scores", "IntermediateName", "Noname");
	new FXLabel(h2, n, 0, LAYOUT_FIX_WIDTH|LAYOUT_FILL_Y|JUSTIFY_LEFT,
	 		0,0,120,0, 0,0,0,0);
	v = getApp()->reg().readUnsignedEntry("Scores", "IntermediateTime", 9999);
	new FXLabel(h2, FXStringVal(v), 0, FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X);

	FXHorizontalFrame *h3 =
        	new FXHorizontalFrame(contents, LAYOUT_SIDE_LEFT|LAYOUT_FILL_X|LAYOUT_FILL_Y,
    	 		0,0,0,0, 5,5,5,5, 0,0);
	new FXLabel(h3, "Advanced", 0, LAYOUT_FIX_WIDTH|LAYOUT_FILL_Y|JUSTIFY_LEFT,
	 		0,0,80,0, 0,0,0,0);
	n = getApp()->reg().readStringEntry("Scores", "AdvancedName", "Noname");
	new FXLabel(h3, n, 0, LAYOUT_FIX_WIDTH|LAYOUT_FILL_Y|JUSTIFY_LEFT,
	 		0,0,120,0, 0,0,0,0);
	v = getApp()->reg().readUnsignedEntry("Scores", "AdvancedTime", 9999);
	new FXLabel(h3, FXStringVal(v), 0, FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X);

	FXHorizontalFrame *h4 =
        	new FXHorizontalFrame(contents, LAYOUT_SIDE_LEFT|LAYOUT_FILL_X|LAYOUT_FILL_Y,
    	 		0,0,0,0, 5,5,5,5, 0,0);
	new FXLabel(h4, "Expert", 0, LAYOUT_FIX_WIDTH|LAYOUT_FILL_Y|JUSTIFY_LEFT,
	 		0,0,80,0, 0,0,0,0);
	n = getApp()->reg().readStringEntry("Scores", "ExpertName", "Noname");
	new FXLabel(h4, n, 0, LAYOUT_FIX_WIDTH|LAYOUT_FILL_Y|JUSTIFY_LEFT,
	 		0,0,120,0, 0,0,0,0);
	v = getApp()->reg().readUnsignedEntry("Scores", "ExpertTime", 9999);
	new FXLabel(h4, FXStringVal(v), 0, FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X);

      	new FXHorizontalSeparator(contents, SEPARATOR_NONE|LAYOUT_FILL_X,
		0,0,0,0, 1,1, 5,5);

        FXButton *button = new FXButton(contents, "OK", 0, &dlg,
	        FXDialogBox::ID_ACCEPT, BUTTON_INITIAL|BUTTON_DEFAULT|FRAME_RAISED|FRAME_THICK|LAYOUT_CENTER_X,
         	0,0,0,0, 32,32,5,5);

        button->setFocus();
      	dlg.execute(PLACEMENT_OWNER);

	return 1;
}

long MainWindow::onCmdIconify(FXObject *, FXSelector, void *)
{
	iconify();
	return 1;
}

long MainWindow::onCmdQuit(FXObject *, FXSelector, void *)
{
	getApp()->exit(0);
 	return 1;
}

long MainWindow::onCmdSkill(FXObject *, FXSelector, void *)
{
	if (game->engine.game_skill != skill)
	  	game->engine.newGame((Skill)skill);
	return 1;
}

long MainWindow::onCmdSound(FXObject *, FXSelector, void *)
{
	game->engine.sounds = !game->engine.sounds;
	return 1;
}

long MainWindow::onCmdAbout(FXObject *, FXSelector, void *)
{
	FXDialogBox about(this, "About FX-Zacman", DECOR_TITLE|DECOR_BORDER,
 		0,0,0,0, 0,0,0,0, 0,0);

  	FXVerticalFrame *info2 =
        	new FXVerticalFrame(&about, LAYOUT_SIDE_LEFT|LAYOUT_FILL_X|LAYOUT_FILL_Y,
       	 		0,0,0,0, 10,10,10,10, 0,0);
  	FXHorizontalFrame *cont =
        	new FXHorizontalFrame(info2, LAYOUT_SIDE_RIGHT|LAYOUT_FILL_X|LAYOUT_FILL_Y,
       	 		0,0,0,0, 0,0,0,0, 0,0);
  	FXHorizontalFrame *infoandicon =
        	new FXHorizontalFrame(cont, LAYOUT_SIDE_RIGHT|LAYOUT_FILL_X|LAYOUT_FILL_Y,
       	 		0,0,0,0, 0,0,0,0, 0,0);
	new FXLabel(infoandicon, FXString::null, bigicon, LAYOUT_FILL_X,
 		0,0,0,0, 0,0,0,0);
  	FXVerticalFrame *info =
   		new FXVerticalFrame(infoandicon, LAYOUT_SIDE_RIGHT|LAYOUT_FILL_X|LAYOUT_FILL_Y,
   			0,0,0,0, 10,10,10,10, 0,0);
    	new FXLabel(info, "FX-Zacman " VERSION "\n", 0, JUSTIFY_LEFT|ICON_BEFORE_TEXT|LAYOUT_FILL_X);
      	new FXHorizontalSeparator(info, SEPARATOR_GROOVE|LAYOUT_FILL_X);
        new FXLabel(info,
        	"\nCopyright "
#ifdef WIN32
        	""
#else
         "(C)"
#endif
         " 2002 Sandro Sigala <sandro@sigala.it>.\n\n"
        	"Latest version can be found at http://www.sigala.it/sandro.",
      		0, JUSTIFY_LEFT|LAYOUT_FILL_X|LAYOUT_FILL_Y);
        FXButton *button = new FXButton(cont, "OK", 0, &about,
	        FXDialogBox::ID_ACCEPT, BUTTON_INITIAL|BUTTON_DEFAULT|FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT,
         	0,0,0,0, 32,32,5,5);

  	FILE *fp;
      	if ((fp = fopen(PATH_DATA_DIR "/Copying", "r")) != 0) {
  	      	FXuint size, n;
  	      	FXchar *text;
  	      	fseek(fp, 0, SEEK_END);
    	      	size = ftell(fp);
  	      	fseek(fp, 0, SEEK_SET);
      	      	FXCALLOC(&text,FXchar,size+1);
      	      	n = fread(text, 1, size, fp);
      	      	fclose(fp);
        	new FXLabel(info2, "License:", 0, LAYOUT_SIDE_TOP|JUSTIFY_LEFT|LAYOUT_FILL_X|LAYOUT_FILL_Y);
        	FXVerticalFrame *textf =
        		new FXVerticalFrame(info2, FRAME_SUNKEN|LAYOUT_SIDE_TOP|LAYOUT_FIX_HEIGHT|LAYOUT_FILL_X,
       	 			0,0,0,200, 0,0,0,0, 0,0);
                FXText *textv = new FXText(textf, 0, 0, LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y);
      	      	textv->setText(text, n);
                textv->setEditable(false);
      	      	FXFREE(&text);
        }

        new FXLabel(info2,
        	"\nThis software uses the FOX Toolkit Library (http://www.fox-toolkit.org).",
      		0, JUSTIFY_LEFT|LAYOUT_FILL_X|LAYOUT_FILL_Y);

        button->setFocus();
      	about.execute(PLACEMENT_OWNER);

	return 1;
}

long MainWindow::onCmdUp(FXObject *, FXSelector, void *ptr)
{
	if (game->engine.status == Status_Playing) {
  		FXEvent *event = (FXEvent *)ptr;
		if (event->time - lastkeytime >= game_keytime) {
			game->engine.goUp();
			game->update();
			lastkeytime = event->time;
		}
	}
	return 1;
}

long MainWindow::onCmdDown(FXObject *, FXSelector, void *ptr)
{
	if (game->engine.status == Status_Playing) {
  		FXEvent *event = (FXEvent *)ptr;
		if (event->time - lastkeytime >= game_keytime) {
			game->engine.goDown();
			game->update();
			lastkeytime = event->time;
		}
	}
	return 1;
}

long MainWindow::onCmdLeft(FXObject *, FXSelector, void *ptr)
{
	if (game->engine.status == Status_Playing) {
  		FXEvent *event = (FXEvent *)ptr;
		if (event->time - lastkeytime >= game_keytime) {
			game->engine.goLeft();
			game->update();
			lastkeytime = event->time;
		}
	}
	return 1;
}

long MainWindow::onCmdRight(FXObject *, FXSelector, void *ptr)
{
	if (game->engine.status == Status_Playing) {
  		FXEvent *event = (FXEvent *)ptr;
		if (event->time - lastkeytime >= game_keytime) {
			game->engine.goRight();
			game->update();
			lastkeytime = event->time;
		}
	}
	return 1;
}

long MainWindow::onTimer(FXObject *, FXSelector, void *)
{
	if (game->engine.status == Status_Finished) {
  		FXMessageBox::information(this, MBOX_OK, "FX-Zacman", "Game finished!");

		FXString name;
		switch (game->engine.game_skill) {
  		case Skill_Beginner:
			name = "Beginner";
			break;
  		case Skill_Intermediate:
			name = "Intermediate";
			break;
  		case Skill_Advanced:
			name = "Advanced";
			break;
  		case Skill_Expert:
			name = "Expert";
			break;
		}

		FXString entryn = name + "Name";
		FXString entryt = name + "Time";
		int oldtime = getApp()->reg().readUnsignedEntry("Scores", entryt.text(), 9999);
		int newtime = game->engine.game_time/1000;

		if (newtime < oldtime) {
   			static FXString uname;
			if (FXInputDialog::getString(uname, this, "New FX-Zacman Record!", "Enter your name:")) {
    				uname = uname.trim();
				if (uname.empty())
					uname = "Joe Doe";
				getApp()->reg().writeStringEntry("Scores", entryn.text(), uname.text());
				getApp()->reg().writeUnsignedEntry("Scores", entryt.text(), newtime);
			}
  		}

		game->engine.newGame((Skill)game->engine.game_skill);
	}

	game->engine.timer(100);
	game->update();
	if (game->engine.status == Status_Playing) {
		FXString s;
    		static int last_time = -1, last_pills = -1;
    		if (last_time != game->engine.game_time) {
			s.format(" %d ", game->engine.game_time / 1000);
			time_label->setText(s);
			last_time = game->engine.game_time;
		}
    		if (last_pills != game->engine.num_pills) {
			s.format(" %d ", game->engine.num_pills);
			pills_label->setText(s);
			last_pills = game->engine.num_pills;
		}
  	}
	timerhandle = getApp()->addTimeout(100, this, ID_TIMER);
	return 1;
}

